function distNeuronsToData = dist_rel_neurons_to_data( Distances, cNeurons, varargin )
%
% Determines dissimilarities between relational neurons and data on which they are defined
%
% Distances ... (m x m)-Matrix of dissimilarities between data points
% cNeurons ... Coefficient (n x m)-Matrix of relational neurons
%
% Use squared distances to compare with other methods on euclidean distances !
%
% A.Hasenfuss (c) 2007-2008 (Optimized Version)
% modified for single point handling Frank-Michael Schleif (c) 2011

[n,m] = size( cNeurons );
if(nargin < 3)
    % Determine distances between neurons and data
  temp1 = cNeurons * Distances;
 
  temp2 = zeros(n,1);
  for i = 1:n
    temp2(i,1) = 0.5 * (temp1(i,:) * cNeurons(i,:)');
  end
 
  distNeuronsToData = temp1 - temp2*ones(1,m);
else
    iSampleIndex = varargin{1};
    temp1 = cNeurons * Distances(:,iSampleIndex);
    %distNeuronsToData = temp1 - 0.5 * diag(cNeurons * Distances * cNeurons');
    distNeuronsToData = bsxfun(@minus,temp1, 0.5 * sum(bsxfun(@times,(cNeurons * Distances),cNeurons),2));
end