function distNeuronsToData = dist_rel_neurons_to_data_ny(C,W,Cp, cNeurons, varargin )
%
% Determines dissimilarities between relational neurons and data on which they are defined
%
% Distances ... (m x m)-Matrix of dissimilarities between data points
% cNeurons ... Coefficient (n x m)-Matrix of relational neurons
%
% Use squared distances to compare with other methods on euclidean distances !
%
% A.Hasenfuss (c) 2007-2008 (Optimized Version)
% modified for single point handling Frank-Michael Schleif (c) 2011
%
% Distances = C * W * Cp; 
[n,m] = size( cNeurons );
if(nargin < 5)
    distNeuronsToData = zeros(n,m);
    for(k=1:n)
        dist_tmp=((cNeurons(k,:)*C)*W)*Cp;
        distNeuronsToData(k,:) = dist_tmp - 1/2 * dist_tmp * cNeurons(k,:)';
    end
else
    iSampleIndex = varargin{1};
	distNeuronsToData = zeros(n,1);
	if(n>300)
          for(k=1:n)
            dist_tmp=((cNeurons(k,:)*C)*W)*Cp;
            distNeuronsToData(k) = dist_tmp(iSampleIndex) - 1/2 * dist_tmp * cNeurons(k,:)';        
          end
	else
          dist_tmp=((cNeurons*C)*W)*Cp;
          f=@(x)(dist_tmp(x,:)*cNeurons(x,:)');
          distNeuronsToData = dist_tmp(:,iSampleIndex)-0.5* arrayfun(f,[1:n])';
    end
end