function D=fCheckDissimilarity(D)
[n m] = size(D);
del = 10*eps(class(D));

% lower triangle form for D, make sure it's a valid dissimilarity matrix
if n == 1
    n = ceil(sqrt(2*m)); % (1+sqrt(1+8*m))/2, but works for large m
    if n*(n-1)/2 == m & all(D >= 0)
        D = squareform(D); % assumes zero diagonal, similarity not allowed
        warning('matrix has been corrected')
    else
        error('stats:cmdscale:BadDistance',...
              'Not a valid dissimilarity or distance matrix.');
    end
    
% full matrix form, make sure it's valid similarity/dissimilarity matrix
elseif n == m & all(all(D >= 0 & abs(D - D') <= del*max(max(D))))
    
    % it's a dissimilarity matrix
    if all(diag(D) < del)
        % nothing to do
        
    % it's a similarity matrix -- transform to dissimilarity matrix.
    % the sqrt is not entirely arbitrary, see Seber, eqn. 5.73
    elseif all(abs(diag(D) - 1) < del) & all(all(D < 1+del))
        D = sqrt(1 - D);
        warning('matrix has been corrected')
    else
        error('stats:cmdscale:BadDistance',...
              'Not a valid dissimilarity or similarity matrix.')
    end
else
    error('stats:cmdscale:BadDistance',...
          'Not a valid dissimilarity or distance matrix.')
end

