function [f G]  = rglvq_optfun(variables, fDist, fDistGrad,LabelEqualsPrototype)
% [f G] = rglvq_optfun(variables) 
% function to be optimized with - relational - generalized learning vector quantization 
%
% Input:
%       variables            = [alpha_matrix] - see rglvq
%       fDist                = function pointer to distance calculation (with range)
%       fDistGrad            = function pointer to gradient of the distance calculation to one neuron and a set of points
%       LabelEqualsPrototype = N x M bool matrix - where prototype labels match datapoint labels
%
% Output:
%       f                    = cost function error current model
%       G                    = gradient
%
% Conditions of GNU General Public License, version 2 apply.
% See file 'license-gpl2.txt' enclosed in this package.
% Programs are not for use in critical applications!
% 
% (C) Frank-Michael Schleif
% University of Bielefeld
% fschleif@techfak.uni-bielefeld.de
%
[N,nb_prototypes] = size(LabelEqualsPrototype);
variables = reshape(variables, nb_prototypes,N);
if(1) % on all samples
    vRange               = [1:N];
    mDist                = fDist(variables,vRange)'; 
else % on x% of the samples
    vRange               = randsample(N,ceil(0.3*N));
    mDist                = fDist(variables,vRange)';
    LabelEqualsPrototype = LabelEqualsPrototype(vRange,:);
end

Dwrong = mDist;
Dwrong(LabelEqualsPrototype)    = realmax(class(Dwrong));   % set correct labels impossible
[distwrong pidxwrong]           = min(Dwrong.'); % closest wrong

Dcorrect = mDist;
Dcorrect(~LabelEqualsPrototype) = realmax(class(Dcorrect)); % set wrong labels impossible
[distcorrect pidxcorrect]       = min(Dcorrect.'); % closest correct

distcorrectpluswrong = distcorrect + distwrong;
distcorrectminuswrong = distcorrect - distwrong;
mu = distcorrectminuswrong ./ distcorrectpluswrong;
f  = sum(mu);
if nargout > 1  % gradient needed not just function eval
    G = zeros(size(variables)); % initially no gradient
    for k=1:nb_prototypes  % update all prototypes - but only correct entries
        idxc = (k == pidxcorrect);
        idxw = (k == pidxwrong);    % Ks: idxs where actual prototype is nearest wrong
        parAlphaP     = fDistGrad(variables(k,:),vRange(idxc)); % for all samples
        parAlphaM     = fDistGrad(variables(k,:),vRange(idxw)); % for all samples
        NormalizerP   = max(distcorrectpluswrong(idxc) .^2,realmin);
        NormalizerM   = max(distcorrectpluswrong(idxw) .^2,realmin);
        dwd           = bsxfun(@rdivide,2  * distwrong(idxc) ,   NormalizerP );                       
        dcd           = bsxfun(@rdivide,2  * distcorrect(idxw) , NormalizerM ); 
        G(k,:)        = dcd * parAlphaM' - dwd * parAlphaP'; % this combines both updates - sometimes correct sometimes wrong
        %G(k,:)        = - dwd * parAlphaP';  % update only for correct prototype
    end   
%    G = -G./N .* (1 + .0001 * (rand(size(G))-.5)); % help break symmetries    
    G = -G(:);
end
